# Dense Neural Network

A dense neural network is a type of neural network where the all hidden layers are connected via so-called dense layers.
In mva-trainer dense neural networks are realised using [Keras](https://keras.io/) and [TensorFlow](https://www.tensorflow.org).

# Defining the Architecture

The first step of building a dense neural network is defining its architecture.
For this purpose several tools are in place in mva-trainer.
The architecture of a dense neural network is mostly defined by:

* the number of layers
* the number of nodes in each layer
* the activation functions of the layers
* where regularisation layers such as `dropout` or `batchnormalisation` is applied.

In the `DNNMODEL` block a dense neural network can be easily realised.
In the following we will look at the example of a simple classifier with three layers with 50, 40 and 30 nodes each:

```
DNNMODEL
	Name = "My_Classifier"
	Type = Classification-DNN
	Nodes = 50,40,30
	ActivationFunctions = "relu","tanh","sigmoid"
	BatchNormIndece = 1,2
	DropoutIndece = 2,3
	OutputSize = 1
	OutputActivation = "sigmoid"
```
In the block the name and type of the classifier is defined using the `name` and `type` options.
The `Nodes` option expects a list of integers which (in this example) defines the number of nodes per hidden layer.
The `ActivationFunctions` option then defines which activation functions are used in the hidden layers.
In this example `relu`, `tanh`, and `sigmoid` are used.
`BatchNormIndece` and `DropoutIndece` describe the numbers of hidden layers after which either `dropout` or `batchnormalisation` layers are added for regularisation purposed.
The `OutputSize` and `OutputActivation` options define the shape and activation function of the output layer and hence determine whether the classifier is a binary or multi-class classifier.

# Training parameters

After defining the architecture the training of the network becomes relevant.
For this purpose mva-trainer supports a number of relevant options which can be found in the settings tables. Here only a few parameters and their effects are highlighted.

# `Patience` and `MinDelta`
`Patience` and `MinDelta` are options controlling a concept known as `Early Stopping`.
The idea behing early stopping is that both training and validation loss will gradually decrease over time as the model is trained.
While the model continiues to improve on the training data, the decrease on the validation data will plateau at some point.
Afterwards overtraining begins and the model becomes sensitive to statistical fluctuations in the training set and looses regularisation power.
While `Patience` describes the number of epochs with no improvement after which training will be stopped. `MinDelta` describes the minimum change in the monitored quantity to qualify as an improvement.

![Early Stopping](../img/Early_Stopping.png)

# Hyperparameter optimisation

Hyparameter optimisation is a key aspect for training a good neural network. 
At the moment the hyperparameter optimisation can be performed for deep neural networks.
The following parameters steer the hyperparameter optimisation:

* MaxLayers
* MinLayers
* MaxNodes
* MinNodes
* StepNodes
* SetActivationFunctions

To run hyperparameter optimisation you can use the `Optimise.py` script.
```sh
./scripts/Optimise.py -c <config_path> -cp <HTCondorConfig_path> --RunOption <RunOption> --NNetworks <NNetworks> --RandomSeed <RandomSeed>
```
This script will take the config file specified by `config_path` (and the corresponding options) as a "seed config" and create `NNetworks` number of new randomised (control seed using `RandomSeed`) configs in the `HTCondorConfig_path` directory. Furthermore `.sub` (and `.dag`) files are created.
Three `RunOption`s are available to the user:

| Option | Effect |
| ------ | ------ |
| ConversionAndTraining | In this case a `.dag` file is created in the specified `HTCondorConfig_path` directory. In addition two `.sub` files are created, one for the conversion and one for the training. The files are created in a way that the conversion only runs once per optimisation. To start the optimisation (in this case a conversion followed by the training of all networks) simply submit the `.dag` file to the HTCondor deamon using the `condor_submit_dag` command.|
| Conversion | In this case a `.sub` file is created in the specified `HTCondorConfig_path` directory. To start the conversion (in this case only the conversion is run) simply submit the `.sub` file to the HTCondor deamon using the `condor_submit` command.|
| Training | In this case a `.sub` file is created in the specified `HTCondorConfig_path` directory. To start the training (in this case only the training is run) simply submit the `.sub` file to the HTCondor deamon using the `condor_submit` command. **For this job to work converted files must already exist.**|
