#!/bin/bash                                                                                                                                                                                                

echo -e "SETUP\t Python version and ROOT"
setupATLAS
lsetup "python 3.9.14-x86_64-centos7"
lsetup "root 6.26.08-x86_64-centos7-gcc11-opt"
echo -e "SETUP\t Setting up environmental variables"

# Print current working directory                                                                                                                                                                          
echo "Script executed in: $(pwd)"

# Create virtual environment                                                                                                                                                                               
echo "Creating virtual environment..."
python3 -m venv myenv

# Activate virtual environment                                                                                                                                                                             
source myenv/bin/activate

# Install required packages                                                                                                                                                                                
python3 -m pip install --no-cache-dir  --upgrade pip
 python3 -m pip install --no-cache-dir uproot pandas scikit-learn tables matplotlib pydot torch==1.12.0 pyg-lib torch-scatter torch-sparse torch-cluster torch-spline-conv torch-geometric -f https://data\
.pyg.org/whl/torch-1.12.0+cpu.html
# Check if ROOT is available                                                                                                                                                                               
 if command -v root >/dev/null 2>&1 ; then
     echo "ROOT is available in the environment"
 else
     echo "ROOT is not available in the environment. Please install ROOT and add it to your PATH."
 fi

# Check if packages were installed correctly                                                                                                                                                               
pip freeze | grep -E "uproot|pandas|numpy|matplotlib|tables|scikit-learn|torch"

# Deactivate virtual environment                                                                                                                                                                           
echo "Deactivate virtual environment"
deactivate

